import { Injectable } from '@angular/core';
import { Http,  Headers, RequestOptions } from '@angular/http';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import { CommunityInfo } from './communityInfo';
import {EnvProperties} from '../../utils/properties/env-properties';
import {map} from "rxjs/operators";
import {COOKIE} from "../../login/utils/helper.class";

@Injectable()
export class CommunityService {

    constructor(private http: HttpClient) {
      }

    getCommunity(properties: EnvProperties, url: string) {
        return this.http.get((properties.useCache) ? (properties.cacheUrl + encodeURIComponent(url)) : url)
                        //.map(res => <any> res.json())
                        .pipe(map(res => this.parseCommunity(res)));
    }

    updateCommunity(url: string, community: any) {
        //const headers = new Headers({'Content-Type': 'application/json'});
        //const options = new RequestOptions({headers: headers});

        const options = {
          headers: new HttpHeaders({
            'Content-Type': 'application/json',
          })
        };

        const body = JSON.stringify(community);
        return this.http.post(url, body, options);
                          /*.map(res => res.json())*/
    }

    isCommunityManager(properties: EnvProperties, url: string, manager: string) {
      return this.http.get((properties.useCache) ? (properties.cacheUrl + encodeURIComponent(url)) : url)
                      //.map(res => <any> res.json())
                      .pipe(map(res => this.parseCommunity(res)))
                      .pipe(map(community => community.managers.indexOf(manager) !== -1));
    }

    isRIType(properties: EnvProperties, url: string) {
      return this.http.get((properties.useCache) ? (properties.cacheUrl + encodeURIComponent(url)) : url)
          //.map(res => <any> res.json())
          .pipe(map(res => this.parseCommunity(res)))
          .pipe(map(community => (community && community.type && community.type === 'ri')));
    }

    isCommunityType(properties: EnvProperties, url: string) {
        return this.http.get((properties.useCache) ? (properties.cacheUrl + encodeURIComponent(url)) : url)
            //.map(res => <any> res.json())
            .pipe(map(res => this.parseCommunity(res)))
            .pipe(map(community => (community && community.type && community.type === 'community')));
    }

    isSubscribedToCommunity(pid: string, email: string, url: string) {
      return this.http.get(url + '/community/' + pid + '/subscribers')
                      //.map(res =>  ((<any>res === '') ? {} : <any> res.json()))
                      .pipe(map(res => {
                        if (res['subscribers'] && res['subscribers'] != null) {

                          for (let i = 0; i < res['subscribers'].length; i++ ) {
                            if (res['subscribers'][i] != null && res['subscribers'][i].email === email) {
                              return true;
                            }
                          }
                        }
                        return false;

                      }));
    }

    private parseCommunity(data: any): CommunityInfo {

        const resData = Array.isArray(data) ? data[0] : data;

        const community: CommunityInfo = new CommunityInfo();
        community['title'] = resData.name;
        community['shortTitle'] = resData.shortName;
        community['communityId'] = resData.id;
        community['queryId'] = resData.queryId;
        community['logoUrl'] = resData.logoUrl;
        community['description'] = resData.description;
        community['date'] = resData.creationDate;
        community['zenodoCommunity'] = resData.zenodoCommunity;
        community['status'] = 'all';
        if (resData.hasOwnProperty('status')) {
          community['status'] = resData.status;
          const status = ['all', 'hidden', 'manager'];
          if (status.indexOf(community['status']) === -1) {
            community['status'] = 'hidden';
          }
        }
        if (resData.type != null) {
              community['type'] = resData.type;
        }

        if (resData.managers != null) {
            if (community['managers'] === undefined) {
              community['managers'] = new Array<string>();
            }

            const managers = resData.managers;
            const length = Array.isArray(managers) ? managers.length : 1;

            for (let i = 0; i < length; i++) {
              const manager = Array.isArray(managers) ? managers[i] : managers;
              community.managers[i] = manager;
            }
        }

        if (resData.subjects != null) {
            if (community['subjects'] === undefined) {
              community['subjects'] = new Array<string>();
            }

            const subjects = resData.subjects;
            const length = Array.isArray(subjects) ? subjects.length : 1;

            for (let i = 0; i < length; i++) {
              const subject = Array.isArray(subjects) ? subjects[i] : subjects;
              community.subjects[i] = subject;
            }
        }
        return community;
    }

}
