
import {filter, mergeMap} from 'rxjs/operators';
import { Injectable } from '@angular/core';
import {
  Router,
  CanActivate,
  ActivatedRouteSnapshot,
  RouterStateSnapshot,
  CanLoad,
  Route, UrlSegment
} from '@angular/router';
import {Observable} from 'rxjs';
import {CommunityService} from '../community/community.service';
import { EnvironmentSpecificService} from '../../utils/properties/environment-specific.service';
import {ConnectHelper} from '../connectHelper';

@Injectable()
export class ConnectRIGuard implements  CanActivate, CanLoad {

  constructor(private router: Router,
              private communityService: CommunityService,
              private propertiesService: EnvironmentSpecificService) {
  }

  check(community: string): Observable<boolean> | boolean {
    const obs = this.propertiesService.subscribeEnvironment().pipe(mergeMap(properties => {
      return this.communityService.isRIType(properties, properties['communityAPI'] + community);
    }));
    obs.pipe(filter(enabled => !enabled))
        .subscribe(() => this.router.navigate(['errorcommunity']));
    return obs;
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(route.queryParams['communityId']);
  }

  canLoad(route: Route, segments: UrlSegment[]): Observable<boolean> | Promise<boolean> | boolean {
    const path = '/' + route.path + document.location.search;
    return this.check(ConnectHelper.getCommunityFromPath(path));
  }
}
