import {Component, Input} from '@angular/core';
import {SearchResult}     from '../utils/entities/searchResult';
import {ErrorCodes} from '../utils/properties/errorCodes';
import {RouterHelper} from '../utils/routerHelper.class';
import {EnvProperties} from '../utils/properties/env-properties';
import {ZenodoInformationClass} from "./utils/zenodoInformation.class";
import {ActivatedRoute} from "@angular/router";

@Component({
  selector: 'deposit-result',
  templateUrl:'searchResultsInDeposit.component.html'
})

export class SearchResultsInDepositComponent {
  @Input() results: SearchResult[];
  @Input() status: number;
  @Input() type: string;
  @Input() properties:EnvProperties;


  public urlParam: string;
  public linkToAdvancedSearchPage: string;
  public errorCodes:ErrorCodes = new ErrorCodes();
  public routerHelper:RouterHelper = new RouterHelper();
  public errorMessage: string = "No results found";

  @Input() public zenodoInformation: ZenodoInformationClass = new ZenodoInformationClass();

  constructor (private  route: ActivatedRoute) {}

  ngOnInit() {
    if(this.type == "publication") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedPublications;
      this.urlParam = "articleId";
    } else if(this.type == "dataset") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedDatasets;
      this.urlParam = "datasetId";
    } else if(this.type == "software") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedSoftware;
      this.urlParam = "softwareId";
    } else if(this.type == "other") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedOrps;
      this.urlParam = "orpId";
    } else if(this.type == "project") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedProjects;
      this.urlParam = "projectId";
    } else if(this.type == "organization") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedOrganizations;
      this.urlParam = "organizationId";
    } else if(this.type == "dataprovider") {
      this.linkToAdvancedSearchPage = this.properties.searchLinkToAdvancedDataProviders;
      this.urlParam = "datasourceId";
    }

    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;

        if (!this.zenodoInformation) {
          this.zenodoInformation = new ZenodoInformationClass();
        }
        if (!this.zenodoInformation.shareInZenodoUrl) {
          this.zenodoInformation.url = this.properties.zenodo;
        }
        if (!this.zenodoInformation.name) {
          this.zenodoInformation.name = "Zenodo";
        }
      });
  }

  public  quote(params: string):string {
    return '"'+params+'"';
  }
}
