import {Component, ElementRef, Inject, Input, PLATFORM_ID, ViewChild} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {Meta, Title} from '@angular/platform-browser';

import {EnvProperties} from '../../utils/properties/env-properties';
import {ResultLandingInfo} from '../../utils/entities/resultLandingInfo';
import {RouterHelper} from '../../utils/routerHelper.class';

import {PiwikService} from '../../utils/piwik/piwik.service';
import {ResultLandingService} from './resultLanding.service';
import {SEOService} from '../../sharedComponents/SEO/SEO.service';
import {HelperFunctions} from '../../utils/HelperFunctions.class';
import {HelperService} from '../../utils/helper/helper.service';
import {Location} from "@angular/common";
import {LoginErrorCodes} from "../../login/utils/guardHelper.class";


@Component({
  selector: 'result-landing',
  templateUrl: 'resultLanding.component.html',

})

export class ResultLandingComponent {
  @Input() type: string = "publication";
  @Input() piwikSiteId = null;
  @Input() communityId = null;

  // Active tab variable for responsiveness
  @Input() activeTab: string = "References";

  @ViewChild('AlertModalDeletedByInference') alertModalDeletedByInference;
  public deleteByInferenceOpened: boolean = false;

  public resultLandingInfo: ResultLandingInfo;
  public id: string;
  public title: string;

  // Links for SEO
  public linkToLandingPage: string = null;
  public linkToSearchPage: string = null;

  // APP BOX variables
  public showAllCollectedFrom: boolean = false;
  public showAllDownloadFrom: boolean = false;

  public thresholdDescription: number = 270;
  public showNumDescription: number = 270;

  // Metrics tab variables
  public metricsClicked: boolean;
  public viewsFrameUrl: string;
  public downloadsFrameUrl: string;
  public totalViews: number;
  public totalDownloads: number;
  public pageViews: number;

  // Custom tab paging variables
  public referencesPage: number = 1;
  public organizationsPage: number = 1;
  public softwarePage: number = 1;
  public bioentitiesPage: number = 1;
  public openCitationsPage: number = 1;
  public pageSize: number = 10;

  // Map counting variables
  public bioentitiesNum: number = 0;
  public relatedResearchResultsNum: number = 0;

  // Message variables
  public warningMessage = "";
  public errorMessage = "";
  public showLoading: boolean = true;

  public routerHelper: RouterHelper = new RouterHelper();

  private doi: string;
  public doiURL: string;
  private result;
  sub: any;
  piwiksub: any;
  infoSub: any;
  properties: EnvProperties;
  public pageContents = null;
  public divContents = null;

  @ViewChild('accordions')
  accordions: ElementRef;
  testBrowser: boolean = false;

  constructor(private _resultLaningService: ResultLandingService,
              private _piwikService: PiwikService,
              private route: ActivatedRoute,
              private router: Router,
              private _meta: Meta,
              private _title: Title,
              private _router: Router,
              private helper: HelperService,
              private seoService: SEOService,
              private _location: Location) {
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;
        //this.getDivContents();
        this.getPageContents();
        this.doiURL = this.properties.doiURL;
        this.updateUrl(data.envSpecific.baseLink + this._router.url);

        this.sub = this.route.queryParams.subscribe(data => {
          this.resultLandingInfo = null;
          if(this.type == "publication") {
            this.updateTitle("Publication");
            this.linkToLandingPage = this.properties.searchLinkToPublication;
            this.linkToSearchPage = this.properties.searchLinkToPublications;
            this.id = data['articleId'];
            this.title = "Publication";
          } else if(this.type == "dataset") {
            this.updateTitle("Dataset");
            this.linkToLandingPage = this.properties.searchLinkToDataset;
            this.linkToSearchPage = this.properties.searchLinkToDatasets;
            this.id = data['datasetId'];
            this.title = "Research Data";
          } else if(this.type == "software") {
            this.updateTitle("Software");
            this.linkToLandingPage = this.properties.searchLinkToSoftware;
            this.linkToSearchPage = this.properties.searchLinkToSoftwareLanding;
            this.id = data['softwareId'];
            this.title = "Software";
          } else if(this.type == "orp") {
            this.updateTitle("Other Research Product");
            this.linkToLandingPage = this.properties.searchLinkToOrp;
            this.linkToSearchPage = this.properties.searchLinkToOrps;
            this.id = data['orpId'];
            this.title = "Other Research Product";
          }
          this.updateDescription("");

          this.metricsClicked = false;

          if (this.id) {
            this.getProvenanceVocabularyAndResultLandingInfo();
          } else {
            this.showLoading = false;

            this._router.navigate(['/error'], {queryParams: {"page": this._location.path(true), "page_type": this.type}});
            /*
            this.warningMessage = "No valid ";
            if(this.type == "publication" || this.type == "software") {
              this.warningMessage += this.type + " ";
            } else if(this.type == "dataset") {
              this.warningMessage += "research data ";
            } else if(this.type == "orp") {
              this.warningMessage += "other research product ";
            }
            this.warningMessage += "id";
            */
          }

          this.viewsFrameUrl = this.properties.framesAPIURL + 'merge.php?com=query&data=[{"query":"resRepoViews", "resTitle":"' + this.id + '", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":80,"sort":"xaxis","xStyle":{"r":-30,"s":"6","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly views"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true&persistent=false';
          this.downloadsFrameUrl = this.properties.framesAPIURL + 'merge.php?com=query&data=[{"query":"resRepoDownloads", "resTitle":"' + this.id + '", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":80,"sort":"xaxis","xStyle":{"r":-30,"s":"6","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly downloads"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true&persistent=false';

          this.scroll();
        });
      });
  }

  private getPageContents() {
    this.helper.getPageHelpContents(this._router.url, this.properties, this.communityId).subscribe(contents => {
      this.pageContents = contents;
    });
  }

  private getDivContents() {
    this.helper.getDivHelpContents(this._router.url, this.properties, this.communityId).subscribe(contents => {
      this.divContents = contents;
    });
  }

  ngOnDestroy() {
    if (this.sub) {
      this.sub.unsubscribe();
    }
    if (this.piwiksub) {
      this.piwiksub.unsubscribe();
    }
    if (this.infoSub) {
      this.infoSub.unsubscribe();
    }
  }

  private getOpenCitations(id: string) {
    this._resultLaningService.getOpenCitations(this.id, this.properties).subscribe(
      data => {
        this.resultLandingInfo.openCitations = data[1];
      },
      err => {
        this.handleError("Error getting open citation for "+this.type+" with id: " + this.id, err);
      }
    );
  }

  private getProvenanceVocabularyAndResultLandingInfo() {
    this.warningMessage = '';
    this.errorMessage = '';
    this.showLoading = true;

    this.resultLandingInfo = null;

    this._resultLaningService.getProvenanceActionVocabulary(this.properties).subscribe(
      provenanceActionVocabulary => {
        this.getResultLandingInfo(provenanceActionVocabulary);
      }, err => {
        this.getResultLandingInfo(null);
        this.handleError("Error getting provenance action vocabulary for "+this.type+" with id: " + this.id, err);
      }
    );

  }

  private getResultLandingInfo(provenanceActionVocabulary: any) {
    this.infoSub = this._resultLaningService.getResultLandingInfo(this.id, this.type, provenanceActionVocabulary, this.properties).subscribe(
      data => {
        this.resultLandingInfo = data;
        this.seoService.createLinkForCanonicalURL(this.properties.baseLink + this.linkToLandingPage + this.resultLandingInfo.record["result"]["header"]["dri:objIdentifier"]);
        if ((this.type == "publication") && (this.properties.environment == "beta" || this.properties.environment == "development") && (typeof document !== 'undefined')) {
          this.getOpenCitations(this.id);
        }

        if (this.resultLandingInfo.title) {
          this.updateTitle(this.resultLandingInfo.title);
          this.updateDescription((this.resultLandingInfo.description ? (this.resultLandingInfo.description) : ("," + this.resultLandingInfo.title)));
        }
        if (this.properties.enablePiwikTrack && (typeof document !== 'undefined')) {
          this.piwiksub = this._piwikService.trackView(this.properties, this.resultLandingInfo.title/*.name*/, this.piwikSiteId).subscribe();
        }

        let bioentitiesNum = 0;
        if (this.resultLandingInfo.bioentities != undefined) {
          this.resultLandingInfo.bioentities.forEach(function (value, key, map) {
            bioentitiesNum += value.size;
          });
        }
        this.bioentitiesNum = bioentitiesNum;

        let relatedResearchResultsNum = 0;
        if (this.resultLandingInfo.relatedResearchResults != undefined) {
          this.resultLandingInfo.relatedResearchResults.forEach(function (value, key, map) {
            relatedResearchResultsNum += value.length;
          });
        }
        this.relatedResearchResultsNum = relatedResearchResultsNum;

        this.result = {
          id: this.id,
          type: this.type,
          source: "openaire",
          title: this.resultLandingInfo.title,
          url: '',
          result: '',
          accessRights: this.resultLandingInfo.accessMode,
          embargoEndDate: ''
        };

        if (this.resultLandingInfo.identifiers != undefined && this.resultLandingInfo.identifiers.has('doi')) {
          this.doi = this.resultLandingInfo.identifiers.get('doi')[0];
        }

        this.showLoading = false;

        if (this.resultLandingInfo.references) {
          this.activeTab = "References";
        } else if (this.resultLandingInfo.relatedResearchResults) {
          this.activeTab = "Related Research Results";
        } else if (this.resultLandingInfo.similarResearchResults) {
          this.activeTab = "Similar Research Results";
        } else if (this.resultLandingInfo.organizations) {
          this.activeTab = "Related Organizations";
        } else if (this.resultLandingInfo.bioentities) {
          this.activeTab = "Bioentities";
        } else if (this.resultLandingInfo.software) {
          this.activeTab = "Software";
        } else {
          this.activeTab = "Metrics";
          //this.metricsClicked = true;
        }
      },
      err => {
        this.handleError("Error getting "+this.type+" for id: " + this.id, err);

        if(err.status == 404) {
          this._router.navigate(['/error'], {queryParams: {"page": this._location.path(true), "page_type": this.type}});
        }

        if(this.type == "publication" || this.type == "software") {
          this.errorMessage = 'No '+this.type+' found';
        } else if(this.type == "dataset") {
          this.errorMessage += "No research data found";
        } else if(this.type == "orp") {
          this.errorMessage += "No research product found";
        }
        this.showLoading = false;
        this.seoService.createLinkForCanonicalURL(this.properties.baseLink + this.linkToSearchPage);
      }
    );
  }

  public metricsResults($event) {
    this.totalViews = $event.totalViews;
    this.totalDownloads = $event.totalDownloads;
    this.pageViews = $event.pageViews;
  }

  public buildCurationTooltip(): string {
    let tooltipContent: string = "<div class='uk-margin uk-padding-small'>";

    tooltipContent += "<h4>Record in preview</h4>";
    tooltipContent += "<p>Bibliographic record accepted by the system, but not yet processed by <br> OpenAIRE tools for information quality improvement and de-duplication</p>";

    tooltipContent += "</div>";
    return tooltipContent;
  }

  private updateDescription(description: string) {
    this._meta.updateTag({content: description.substring(0, 160)}, "name='description'");
    this._meta.updateTag({content: description.substring(0, 160)}, "property='og:description'");
  }

  private updateTitle(title: string) {
    var _prefix = "";
    // if(!this.communityId) {
    //   _prefix = "OpenAIRE | ";
    // }
    // var _title = _prefix + ((title.length > 50) ? title.substring(0, 50) : title);
    this._title.setTitle(title);
    this._meta.updateTag({content: title}, "property='og:title'");
  }

  private updateUrl(url: string) {
    this._meta.updateTag({content: url}, "property='og:url'");
  }

  public totalPages(totalResults: number): number {
    let totalPages: any = totalResults / this.pageSize;
    if (!(Number.isInteger(totalPages))) {
      totalPages = (parseInt(totalPages, this.pageSize) + 1);
    }
    return totalPages;
  }

  public updateReferencesPage($event) {
    this.referencesPage = $event.value;
  }

  public updateOrganizationsPage($event) {
    this.organizationsPage = $event.value;
  }

  public updateSoftwarePage($event) {
    this.softwarePage = $event.value;
  }

  public updateBioentitiesPage($event) {
    this.bioentitiesPage = $event.value;
  }

  public updateOpenCitationsPage($event) {
    this.openCitationsPage = $event.value;
  }

  public keysToArray(bioentities: Map<string, string>): string[] {
    let keys: string[] = [];
    bioentities.forEach(function (value, key, map) {
      keys.push(key);
    });
    return keys;
  }

  public getKeys(map) {
    return Array.from(map.keys());
  }

  public scroll() {
    HelperFunctions.scroll();
  }

  private handleError(message: string, error) {
    if(this.type == "publication") {
      console.error("Publication Landing Page: " + message, error);
    } else if(this.type == "dataset") {
      console.error("Research Data Landing Page: " + message, error);
    } else if(this.type == "software") {
      console.error("Software Landing Page: " + message, error);
    } else if(this.type == "orp") {
      console.error("Other Research Product Landing Page: " + message, error);
    } else {
      console.error("Landing Page: " + message, error);
    }
  }

  isRouteAvailable(routeToCheck: string) {
    for (let i = 0; i < this.router.config.length; i++) {
      let routePath: string = this.router.config[i].path;
      if (routePath == routeToCheck) {
        return true;
      }
    }
    return false;
  }

  openDeletedByInference() {
    this.deleteByInferenceOpened = true;
    this.alertModalDeletedByInference.cancelButton = false;
    this.alertModalDeletedByInference.okButton = false;
    this.alertModalDeletedByInference.alertTitle = "Original sources";
    //this.alertModalDeletedByInference.message = "There was an error in csv downloading. Please try again later.";
    //this.alertModalDeletedByInference.okButtonText = "OK";
    this.alertModalDeletedByInference.open();
  }

  gotoAccordions() {
    //this.accordions.nativeElement.scrollIntoView();

    let offsetHeight = document.getElementById('stickyNavbar').offsetHeight;

    // scroll to your element
    this.accordions.nativeElement.scrollIntoView(true);

    // now account for fixed header
    var scrolledY = window.scrollY;

    if(scrolledY){
      window.scroll(0, scrolledY - offsetHeight - 50);
    }
  }
}
