import {Injectable} from '@angular/core';
import {
  Router,
  CanActivate,
  ActivatedRouteSnapshot,
  RouterStateSnapshot,
  CanLoad,
  Route,
  UrlSegment
} from '@angular/router';
import {Observable} from 'rxjs';
import {Session} from './utils/helper.class';
import {LoginErrorCodes} from './utils/guardHelper.class';
import {UserManagementService} from "../services/user-management.service";
import {EnvironmentSpecificService} from "../utils/properties/environment-specific.service";
import {filter, map, mergeMap} from "rxjs/operators";

@Injectable()
export class AdminLoginGuard implements CanActivate{

  constructor(private router: Router,
              private userManagementService: UserManagementService,
              private propertiesService: EnvironmentSpecificService) {
  }

  check(path: string): Observable<boolean>  {
    let errorCode = LoginErrorCodes.NOT_LOGIN;
    const obs = this.propertiesService.subscribeEnvironment().pipe(mergeMap(properties => {
      return this.userManagementService.getUserInfo(properties['userInfoUrl']).pipe(map( user => {
        if(user) {
          errorCode = LoginErrorCodes.NOT_ADMIN;
        }
        return Session.isPortalAdministrator(user);
      }));
    }));
    obs.pipe(filter( isAdmin => !isAdmin)).subscribe( () => {
      this.router.navigate(['/user-info'], {
        queryParams: {
          'errorCode': errorCode,
          'redirectUrl': path
        }
      });
    });
    return obs;
  }

 /* check(path: string): boolean {
    let loggedIn = false;
    let isAdmin = false;
    let errorCode = LoginErrorCodes.NOT_LOGIN;
    if (Session.isLoggedIn()) {
      loggedIn = true;
      isAdmin = Session.isPortalAdministrator();
      if (!isAdmin) {
        errorCode = LoginErrorCodes.NOT_ADMIN;
      }
    }
    if (!loggedIn || !isAdmin) {
      this.router.navigate(['/user-info'], {
        queryParams: {
          'errorCode': errorCode,
          'redirectUrl': path
        }
      });
    }
    return loggedIn && isAdmin;
  }*/

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(state.url);
  }

}
