import {Component, Input, ViewChild} from '@angular/core';
import {Observable}       from 'rxjs';
import { Router, ActivatedRoute} from '@angular/router';
import {Filter, Value,AdvancedField} from '../searchUtils/searchHelperClasses.class';
import {SearchProjectsService} from '../../services/searchProjects.service';
import {SearchResult}     from '../../utils/entities/searchResult';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {ErrorMessagesComponent}    from '../../utils/errorMessages.component';
import {AdvancedSearchPageComponent} from '../searchUtils/advancedSearchPage.component';
import {SearchFields, FieldDetails} from '../../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from '../searchUtils/searchUtils.class';
import{EnvProperties} from '../../utils/properties/env-properties';

@Component({
    selector: 'advanced-search-projects',
    template: `
    <advanced-search-page pageTitle="Advanced Search for Projects" entityType="project"
                 type = "projects"
                 [(results)] = "results"
                 [(searchUtils)] = "searchUtils"
                 [(fieldIds)]="fieldIds" [(fieldIdsMap)]="fieldIdsMap"   [(selectedFields)]="selectedFields"
                 (queryChange)="queryChanged($event)"
                 [csvParams]="csvParams" csvPath="projects" simpleSearchLink="/search/find/projects"
                 [disableForms]="disableForms"
                 [loadPaging]="loadPaging"
                 [oldTotalResults]="oldTotalResults"  [piwikSiteId]=piwikSiteId
                 searchFormClass="projectsSearchForm">
    </advanced-search-page>

    `
 })

export class AdvancedSearchProjectsComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
@Input() piwikSiteId = null;
@Input() customFilter:SearchCustomFilter= null;
  public results =[];
  public filters =[];

  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public searchFields:SearchFields = new SearchFields();

  public fieldIds:  string[] = this.searchFields.PROJECT_ADVANCED_FIELDS;
  public fieldIdsMap = this.searchFields.PROJECT_FIELDS;
  public selectedFields:AdvancedField[] =  [];
  properties:EnvProperties;

  @ViewChild (AdvancedSearchPageComponent) searchPage : AdvancedSearchPageComponent ;
  public resourcesQuery = "(oaftype exact project)";
  public csvParams: string;
  public disableForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  public pagingLimit: number = 0;
  public isPiwikEnabled;

  constructor (private route: ActivatedRoute, private _searchProjectsService: SearchProjectsService ) {

    this.results =[];
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;




  }
  ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties= data.envSpecific;
        this.searchUtils.baseUrl = data.envSpecific.searchLinkToAdvancedProjects;
         this.pagingLimit = data.envSpecific.pagingLimit;
         this.isPiwikEnabled = data.envSpecific.enablePiwikTrack;

      });
    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;
    this.sub =  this.route.queryParams.subscribe(params => {
      if(params['page'] && this.searchUtils.page != params['page']) {
        this.loadPaging = false;
        this.oldTotalResults = this.searchUtils.totalResults;
      }

      let page = (params['page']=== undefined)?1:+params['page'];
      this.searchUtils.page = ( page <= 0 ) ? 1 : page;

      this.searchUtils.size = (params['size']=== undefined)?10:+params['size'];
      if(this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }

      this.searchPage.fieldIds = this.fieldIds;
      this.selectedFields =[];
      this.searchPage.selectedFields = this.selectedFields;
      this.searchPage.fieldIdsMap = this.fieldIdsMap;
      this.searchPage.customFilter = this.customFilter;
      this.searchPage.getSelectedFiltersFromUrl(params);
      this.getResults(this.searchPage.createQueryParameters(),  this.searchUtils.page, this.searchUtils.size);

    });
  }
  ngOnDestroy() {
    this.sub.unsubscribe();
  }
  sub: any;
  public getResults(parameters:string, page: number, size: number){
    if(page > this.pagingLimit) {
      size=0;
    }
    if(page <= this.pagingLimit || this.searchUtils.status == this.errorCodes.LOADING) {
      if(parameters!= null && parameters != ''  ) {
        this.csvParams ="&fq=( "+this.resourcesQuery + "and (" + parameters + "))";
      }else{
        this.csvParams ="&fq="+this.resourcesQuery;
      }

      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;
      //this.searchPage.openLoading();
      this.disableForms = true;
      this.results = [];
      this.searchUtils.totalResults = 0;

      //console.info("Advanced Search for Publications: Execute search query "+parameters);
       this._searchProjectsService.advancedSearchProjects(parameters,  page, size, this.properties).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              this.results = data[1];
              this.searchPage.updateBaseUrlWithParameters();
              //var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
              //this.searchPage.closeLoading();
              this.disableForms = false;

              if(this.searchUtils.status == this.errorCodes.DONE) {
                // Page out of limit!!!
                let totalPages:any = this.searchUtils.totalResults/(this.searchUtils.size);
                if(!(Number.isInteger(totalPages))) {
                    totalPages = (parseInt(totalPages, 10) + 1);
                }
                if(totalPages < page) {
                  this.searchUtils.totalResults = 0;
                  this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
                }
              }
          },
          err => {
              //console.log(err);
              this.handleError("Error getting projects", err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

              //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = errorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.NOT_AVAILABLE;
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/

              //this.searchPage.closeLoading();
              this.disableForms = false;

          }
      );
    }
  }
  private setFilters(){
    //TODO set filters from
  }

  public queryChanged($event) {
    this.loadPaging = true;

    var parameters = $event.value;
    //this.getResults(parameters, this.searchUtils.page,this.searchUtils.size);
    //console.info("queryChanged: Execute search query "+parameters);

  }

  private handleError(message: string, error) {
    console.error("Projects advanced Search Page: "+message, error);
  }
}
