import {Component, Input, ViewChild} from '@angular/core';
import { ActivatedRoute} from '@angular/router';

import { Filter, Value} from '../searchUtils/searchHelperClasses.class';

import {SearchDataprovidersService} from '../../services/searchDataproviders.service';
import {SearchResult}     from '../../utils/entities/searchResult';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {ErrorMessagesComponent}    from '../../utils/errorMessages.component';
import {SearchFields, FieldDetails} from '../../utils/properties/searchFields';
import {SearchPageTableViewComponent } from '../searchUtils/searchPageTableView.component';
import {SearchUtilsClass } from '../searchUtils/searchUtils.class';
import{EnvProperties} from '../../utils/properties/env-properties';

@Component({
    selector: 'search-journals-table',
    template: `

    <search-page-table    pageTitle="OpenAIRE Journals Table"
                          type="content providers" entityType="dataprovider" [(filters)] = "filters"
                          [(results)] = "results"  [(searchUtils)] = "searchUtils"
                          [columnNames]="columnNames"
                          [showResultCount]=false
                          [disableForms]="disableForms"
                          [enableSearchView]="enableSearchView"
                          [searchViewLink]="'/search/journals'"
                          searchFormClass="journalsTableSearchForm"
                          formPlaceholderText="Search for Journals"
                          [mapUrl]="mapUrl" [mapTooltipType]="'journals'">
    </search-page-table>
    `

})
export class SearchJournalsTableComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
  public results =[];
  public filters =[];
  public columnNames = ["Name", "Type", "Country", "Institution", "Compatibility"];
  public baseUrl:string;
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public sub: any; public subResults: any;
  public _location:Location;
  public searchFields:SearchFields = new SearchFields();
  public refineFields: string[] =  this.searchFields.JOURNAL_FIELDS;
  public mapUrl: string;
properties:EnvProperties;
  /*public CSV: any = {  "columnNames":  [ "Title", "Type", "Coutries", "Compatibility" ],
                        "export":[]
                     };
  public CSVDownloaded = false;
  public csvParams: string;
  public resourcesQuery = "&query=((oaftype exact datasource) not(datasourcecompatibilityid = UNKNOWN) not(datasourcecompatibilityid = notCompatible) not(datasourcetypeuiid = other))";*/

  public disableForms: boolean = false;
  public enableSearchView: boolean = true;

  @ViewChild (SearchPageTableViewComponent) searchPage : SearchPageTableViewComponent ;

  constructor (private route: ActivatedRoute, private _searchDataprovidersService: SearchDataprovidersService ) {
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;
  }

  public ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;
        this.baseUrl = data.envSpecific.searchLinkToJournalsTable;
        this.mapUrl = this.properties.statisticsFrameAPIURL+"markers-test.html?show=journals";
      });
    this.searchPage.refineFields = this.refineFields;
    this.sub =  this.route.queryParams.subscribe(params => {
      this.searchUtils.keyword = (params['keyword']?params['keyword']:'');
      //this.searchUtils.page = (params['page']=== undefined)?1:+params['page'];
      this.filters = this.createFilters();
      this.searchPage.getParametersFromUrl(params);
      this._getResults();
    });
  }

  public ngOnDestroy() {
    if(this.sub){
      this.sub.unsubscribe();
    }
    if(this.subResults){
      this.subResults.unsubscribe();
    }
  }
    private _getResults(){
        //this.csvParams = this.resourcesQuery+"&type=datasources";

      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;
      this.disableForms = true;
      this.enableSearchView = false;
      this.results = [];
      this.searchUtils.totalResults = 0;

      let size: number = 0;
      this.subResults = this._searchDataprovidersService.searchJournalsTable(this.properties).subscribe(
          data => {
              size = data;
              if(size > 0) {
                this.subResults = this._searchDataprovidersService.searchJournals("", null, 1, size, [],this.properties).subscribe(
                    data => {
                        this.searchUtils.totalResults = data[0];
                        this.results = data[1];

                        this.searchPage.checkSelectedFilters(this.filters);

                        //var errorCodes:ErrorCodes = new ErrorCodes();
                        this.searchUtils.status = this.errorCodes.DONE;
                        if(this.searchUtils.totalResults == 0 ){
                          this.searchUtils.status = this.errorCodes.NONE;
                        } else {
                          this.searchPage.triggerInitialLoad();
                          this.searchPage.transform(this.results);
                          this.disableForms = false;
                        }
                        this.enableSearchView = true;
                    },
                    err => {
                        //console.log(err);
                        this.handleError("Error getting journals", err);
                        this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

                         //TODO check erros (service not available, bad request)
                        // if( ){
                        //   this.searchUtils.status = ErrorCodes.ERROR;
                        // }
                        //var errorCodes:ErrorCodes = new ErrorCodes();
                        //this.searchUtils.status = errorCodes.ERROR;
                        /*if(err.status == '404') {
                          this.searchUtils.status = this.errorCodes.NOT_FOUND;
                        } else if(err.status == '500') {
                          this.searchUtils.status = this.errorCodes.ERROR;
                        } else {
                          this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
                        }*/

                        this.enableSearchView = true;
                    }
                );
              } else {
                this.searchPage.checkSelectedFilters(this.filters);

                //var errorCodes:ErrorCodes = new ErrorCodes();
                this.searchUtils.status = this.errorCodes.NONE;
                this.enableSearchView = true;
              }
          },
          err => {
              //console.log(err);
              this.handleError("Error getting journals (size=0)", err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

               //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = ErrorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.ERROR;
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
              this.enableSearchView = true;
          }
      );

    }
  private setFilters(){
    //TODO set filters from
  }

  private createFilters():Filter[] {
    var filter_names=["Type","Compatibility Level"];
    var filter_ids=["datasourcetypeuiid","datasourcecompatibilityname"];
    var searchFields = new SearchFields();
    var filter_original_ids = searchFields.JOURNAL_FIELDS;
    var value_names=[
      /*[
      "Institutional Publication Repository","Thematic Publication Repository", "Other Publication Repository",
     "Institutional Repositories Aggregators",
     "Thematic Repositories Aggregators", "Other Repositories Aggregators",
      "Data Repositories", "Data Repositories Aggregators", "Journals", "Journals Aggregators", "CRIS Systems", "Publication Catalogues"],
      */
      ["Journal", "Journal Aggregator\/Publisher"],




      ["OpenAIRE Basic (DRIVER OA)","OpenAIRE 2.0 (EC funding)", "OpenAIRE 2.0+ (DRIVER OA, EC funding)", "OpenAIRE 3.0 (OA, funding)","OpenAIRE Data (funded, referenced datasets)",
       "collected from a compatible aggregator", "proprietary", "under validation"]];

    var value_original_ids=[
      ["pubsrepository::journal", "aggregator::pubsrepository::journals"],
      //["driver","openaire2.0", "driver-openaire2.0", "openaire3.0","openaire2.0_data"]
    ["OpenAIRE Basic (DRIVER OA)","OpenAIRE 2.0 (EC funding)", "OpenAIRE 2.0+ (DRIVER OA, EC funding)", "OpenAIRE 3.0 (OA, funding)","OpenAIRE Data (funded, referenced datasets)",
     "collected from a compatible aggregator", "proprietary", "under validation"]];
    var filters: Filter[] =[];
    for(var i =0 ; i < filter_names.length;i++){
      var values:Value[] = [];
      for(var j =0 ; j < value_names[i].length;j++){
        var value:Value = {name: value_names[i][j], id: value_original_ids[i][j], number:j, selected:false}
        values.push(value);
      }
       var filter:Filter = {title: filter_names[i], filterId: filter_ids[i], originalFilterId:  filter_original_ids[i], values : values, countSelectedValues:0, "filterOperator": 'or', valueIsExact: true, valueIsUnique: false };
       filters.push(filter);
    }
    return filters;
    }

    private handleError(message: string, error) {
      console.error("Journals Search Page - Table view: "+message, error);
    }
}
