import {SearchResearchResultsService} from '../../services/searchResearchResults.service';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {ErrorMessagesComponent}    from '../../utils/errorMessages.component';
import {SearchFields} from '../../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from '../../searchPages/searchUtils/searchUtils.class';
import {DOI} from '../../utils/string-utils.class';
import {Subject} from 'rxjs';
import{EnvProperties} from '../../utils/properties/env-properties';
import {StringUtils} from '../../utils/string-utils.class';

export class FetchResearchResults {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;

  public results =[];

  public requestComplete: Subject<void>;

  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public sub: any;
  public subResults: any;
  public searchFields:SearchFields = new SearchFields();

  public CSV: any = {
    "columnNames":  [
      "Title", "Authors", "Publication Year", "DOI",
      "Funder", "Project Name (GA Number)", "Access"
    ],
    "export":[]
  };
  public CSVDownloaded = false;
  public csvParams: string;

  constructor ( private _searchResearchResultsService: SearchResearchResultsService ) {
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;

    this.requestComplete = new Subject<void>();
  }

  public ngOnDestroy() {
    if(this.sub){
      this.sub.unsubscribe();
    }
    if(this.subResults){
      this.subResults.unsubscribe();
    }
  }

  public getResultsByKeyword(resultType:string, keyword:string,  page: number, size: number, properties:EnvProperties,  customFilter:SearchCustomFilter=null){
    var parameters = "";
    if(keyword.length > 0){
      var DOIs:string[] = DOI.getDOIsFromString(keyword);
      var doisParams = "";

      for(var i =0 ;i < DOIs.length; i++){
        doisParams+=(doisParams.length > 0?"&":"")+'doi="'+ DOIs[i]+'"';
      }
      if(doisParams.length > 0){
        parameters +=  "&"+doisParams;
      }else{
        parameters = "q=" + StringUtils.URIEncode(keyword);
      }
    }

    this.searchUtils.status = this.errorCodes.LOADING;
    var refineParams = null;
    if(customFilter){
      refineParams = (refineParams?(refineParams+'&'):'')+"&fq="+StringUtils.URIEncode(customFilter.queryFieldName + " exact " + StringUtils.quote((customFilter.valueId )));
    }
    this.subResults = this._searchResearchResultsService.search(this.getEntityName(resultType,false), parameters,refineParams, page, size, "", [], properties).subscribe(
      data => {
        this.searchUtils.totalResults = data[0];
        this.results = data[1];

        this.searchUtils.status = this.errorCodes.DONE;
        if(this.searchUtils.totalResults == 0 ){
          this.searchUtils.status = this.errorCodes.NONE;
        }
      },
      err => {
        /*console.log(err);
         //TODO check erros (service not available, bad request)
        // if( ){
        //   this.searchUtils.status = ErrorCodes.ERROR;
        // }
        //var errorCodes:ErrorCodes = new ErrorCodes();
        //this.searchUtils.status = errorCodes.ERROR;
        if(err.status == '404') {
          this.searchUtils.status = this.errorCodes.NOT_FOUND;
        } else if(err.status == '500') {
          this.searchUtils.status = this.errorCodes.ERROR;
        } else {
          this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
        }*/
        this.handleError("Error getting "+this.getEntityName(resultType,true)+" for keyword: "+keyword + (doisParams ? "(DOI)" : ""), err);
        this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
      }
    );
  }

  public getNumForEntity(resultType: string, entity:string, id:string, properties:EnvProperties){
    this.searchUtils.status = this.errorCodes.LOADING;

    if(id != "" && entity != "") {
      this._searchResearchResultsService.numOfEntityResults(this.getEntityName(resultType,false), id, entity, properties).subscribe(
        data => {
          this.searchUtils.totalResults = data;

          this.searchUtils.status = this.errorCodes.DONE;
          if(this.searchUtils.totalResults == 0 ){
            this.searchUtils.status = this.errorCodes.NONE;
          }
        },
        err => {
          /*console.log(err);
           //TODO check erros (service not available, bad request)
          // if( ){
          //   this.searchUtils.status = ErrorCodes.ERROR;
          // }
          //var errorCodes:ErrorCodes = new ErrorCodes();
          //this.searchUtils.status = errorCodes.ERROR;
          if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/
          this.handleError("Error getting "+this.getEntityName(resultType,true)+" for "+entity+" with id: "+id, err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
        }
      );
    }
  }

  public getResultsForEntity(resultType: string, entity:string, id:string, page: number, size: number, properties:EnvProperties){
    this.searchUtils.status = this.errorCodes.LOADING;

    var parameters = "";
    if(entity == "project") {
      parameters = "projects/"+id;
    } else if(entity == "organization") {
      parameters = "organizations/"+id;
    }

    if(parameters != "") {
      this._searchResearchResultsService.searchResultForEntity(this.getEntityName(resultType,false), parameters, page, size, properties).subscribe(
        data => {
          this.searchUtils.totalResults = data[0];
          this.results = data[1];

          this.searchUtils.status = this.errorCodes.DONE;
          if(this.searchUtils.totalResults == 0 ){
            this.searchUtils.status = this.errorCodes.NONE;
          }
        },
        err => {
          /*console.log(err);
           //TODO check erros (service not available, bad request)
          // if( ){
          //   this.searchUtils.status = ErrorCodes.ERROR;
          // }
          //var errorCodes:ErrorCodes = new ErrorCodes();
          //this.searchUtils.status = errorCodes.ERROR;

          if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/
          this.handleError("Error getting "+this.getEntityName(resultType,true)+" for "+entity+" with id: "+id, err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
        }
      );
    }
  }

  public getResultsForDataproviders(resultType: string, id:string, resultsFrom:string, page: number, size: number, properties:EnvProperties){
    this.searchUtils.status = this.errorCodes.LOADING;

    var parameters;
    if(resultsFrom == "collectedFrom") {
      parameters = this.getEntityName(resultType,true)+"?fq=collectedfromdatasourceid exact "+'"'+id+'"';
    } else if(resultsFrom == "hostedBy") {
      parameters = this.getEntityName(resultType,true)+"?fq=resulthostingdatasourceid exact "+'"'+id+'"';
    }

    if(parameters != "") {

      this._searchResearchResultsService.searchForDataproviders(this.getEntityName(resultType,false), parameters, page, size, properties).subscribe(
        data => {
          this.searchUtils.totalResults = data[0];
          this.results = data[1];

          this.searchUtils.status = this.errorCodes.DONE;
          if(this.searchUtils.totalResults == 0 ){
            this.searchUtils.status = this.errorCodes.NONE;
          }
        },
        err => {
          /*console.log(err);
           //TODO check erros (service not available, bad request)
          // if( ){
          //   this.searchUtils.status = ErrorCodes.ERROR;
          // }
          //var errorCodes:ErrorCodes = new ErrorCodes();
          //this.searchUtils.status = errorCodes.ERROR;
          if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/
          this.handleError("Error getting "+this.getEntityName(resultType,true)+" for content provider ("+resultsFrom+") with id: "+id, err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
        }
      );
    }
  }

  public getAggregatorResults(resultType: string, id:string, page: number, size: number, properties:EnvProperties){
    this.searchUtils.status = this.errorCodes.LOADING;

    this.subResults = this._searchResearchResultsService.searchAggregators(this.getEntityName(resultType,false), id, '&fq=collectedfromdatasourceid exact "'+id+'"',"&refine=true&fields=resulthostingdatasource" , page, size, properties).subscribe(
      data => {
        this.results = data;
        this.searchUtils.totalResults = this.results.length;

        this.searchUtils.status = this.errorCodes.DONE;
        if(this.searchUtils.totalResults == 0 ){
          this.searchUtils.status = this.errorCodes.NONE;
        }

        this.requestComplete.complete();
      },
      err => {
        /*console.log(err);
         //TODO check erros (service not available, bad request)
        // if( ){
        //   this.searchUtils.status = ErrorCodes.ERROR;
        // }
        //var errorCodes:ErrorCodes = new ErrorCodes();
        //this.searchUtils.status = errorCodes.ERROR;
        if(err.status == '404') {
          this.searchUtils.status = this.errorCodes.NOT_FOUND;
        } else if(err.status == '500') {
          this.searchUtils.status = this.errorCodes.ERROR;
        } else {
          this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
        }*/
        this.handleError("Error getting "+this.getEntityName(resultType,true)+" for aggregator with id: "+id, err);
        this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

        this.requestComplete.complete();
      }
    );
  }

  private handleError(message: string, error) {
    console.error("Fetch Research Results (class): "+message, error);
  }

  private getEntityName (entityType:string, plural:boolean){
    if(entityType == "publication" ||entityType == "dataset" || entityType == "organization" || entityType == "datasource" || entityType == "project" ){
      if(plural){
        return entityType+ "s";
      }else{
        return entityType;
      }
    }else{
      return entityType;
    }
  }
}
